@@ -5,6 +5,7 @@ import logging
 import copy
 import tempfile
 import shutil
+import numpy as np
 import pandas as pd
 from shapash.webapp.smart_app import SmartApp
 from shapash.utils.io import save_pickle
@@ -15,6 +16,7 @@ from shapash.utils.utils import get_host_name
 from shapash.utils.threading import CustomThread
 from shapash.utils.shap_backend import shap_contributions, check_explainer, get_shap_interaction_values
 from shapash.utils.acv_backend import active_shapley_values, compute_features_import_acv
+from shapash.utils.lime_backend import lime_contributions
 from shapash.utils.check import check_model, check_label_dict, check_ypred, check_contribution_object,\
     check_postprocessing, check_features_name
 from shapash.manipulation.select_lines import keep_right_contributions
@@ -25,6 +27,7 @@ from .multi_decorator import MultiDecorator
 from .smart_plotter import SmartPlotter
 import shapash.explainer.smart_predictor
 from shapash.utils.model import predict_proba, predict
+from shapash.utils.explanation_metrics import find_neighbors, shap_neighbors, get_min_nb_features, get_distance
 
 logging.basicConfig(level=logging.INFO)
 
@@ -90,6 +93,13 @@ class SmartExplainer:
         Dictionary that references the numbers of feature values ​​in the x_pred
     features_imp: pandas.Series (regression) or list (classification)
         Features importance values
+    local_neighbors: dict
+        Dictionary of values to be displayed on the local_neighbors plot.
+        The key is "norm_shap (normalized contributions values of instance and neighbors)
+    features_stability: dict
+        Dictionary of arrays to be displayed on the stability plot.
+        The keys are "amplitude" (average contributions values for selected instances) and
+        "stability" (stability metric across neighborhood)
     preprocessing : category_encoders, ColumnTransformer, list or dict
         The processing apply to the original data.
     postprocessing : dict
@@ -200,7 +210,7 @@ class SmartExplainer:
             }
         backend : str (default: 'shap')
             Select which computation method to use in order to compute contributions
-            and feature importance. Possible values are 'shap' or 'acv'. Default is 'shap'.
+            and feature importance. Possible values are 'shap', 'acv' or 'lime'. Default is 'shap'.
 
         Example
         --------
@@ -221,7 +231,8 @@ class SmartExplainer:
         # Computing contributions using right backend
         if contributions is None:
             if backend.lower() == 'shap':
-                contributions, explainer = shap_contributions(model, self.x_init, self.check_explainer(explainer))
+                contributions, explainer = shap_contributions(
+                    model, self.x_init, self.check_explainer(explainer))
             elif backend.lower() == 'acv':
                 self.backend = 'acv'
                 if features_groups is not None:
@@ -233,8 +244,18 @@ class SmartExplainer:
                     )
                 else:
                     raise NotImplementedError('ACV does not support regression case yet.')
+            elif backend.lower() == 'lime':
+                if features_groups is not None:
+                    raise ValueError('LIME does not support groups of features for now.')
+                else:
+                    contributions = lime_contributions(model=model,
+                                                   x_init=self.x_init,
+                                                   mode=self._case,
+                                                   classes=self._classes, **kwargs)
+
             else:
-                raise ValueError(f'Unknown backend : {backend}. Possible values are "shap" or "acv".')
+                raise ValueError(
+                    f'Unknown backend : {backend}. Possible values are "shap", "acv" or "lime".')
 
         adapt_contrib = self.adapt_contributions(contributions)
         self.state = self.choose_state(adapt_contrib)
@@ -266,6 +287,9 @@ class SmartExplainer:
         self.features_groups = features_groups
         if features_groups:
             self._compile_features_groups(features_groups)
+        self.local_neighbors = None
+        self.features_stability = None
+        self.features_compacity = None
 
     def _compile_features_groups(self, features_groups):
         """
@@ -986,6 +1010,71 @@ class SmartExplainer:
             if self.features_imp is None or force:
                 self.features_imp = self.state.compute_features_import(self.contributions)
 
+    def compute_features_stability(self, selection):
+        """
+        For a selection of instances, compute features stability metrics used in
+        methods `local_neighbors_plot` and `local_stability_plot`.
+        - If selection is a single instance, the method returns the (normalized) contribution values
+        of instance and corresponding neighbors.
+        - If selection represents multiple instances, the method returns the average (normalized) contribution values
+        of instances and neighbors (=amplitude), as well as the variability of those values in the neighborhood (=variability)
+
+        Parameters
+        ----------
+        selection: list
+            Indices of rows to be displayed on the stability plot
+
+        Returns
+        -------
+        Dictionary
+            Values that will be displayed on the graph. Keys are "amplitude", "variability" and "norm_shap"
+        """
+        if (self._case == "classification") and (len(self._classes) > 2):
+            raise AssertionError("Multi-class classification is not supported")
+
+        all_neighbors = find_neighbors(selection, self.x_init, self.model, self._case)
+
+        # Check if entry is a single instance or not
+        if len(selection) == 1:
+            # Compute explanations for instance and neighbors
+            norm_shap, _, _ = shap_neighbors(all_neighbors[0], self.x_init, self.contributions, self._case)
+            self.local_neighbors = {"norm_shap": norm_shap}
+        else:
+            numb_expl = len(selection)
+            amplitude = np.zeros((numb_expl, self.x_pred.shape[1]))
+            variability = np.zeros((numb_expl, self.x_pred.shape[1]))
+            # For each instance (+ neighbors), compute explanation
+            for i in range(numb_expl):
+                (_, variability[i, :], amplitude[i, :],) = shap_neighbors(all_neighbors[i], self.x_init, self.contributions, self._case)
+            self.features_stability = {"variability": variability, "amplitude": amplitude}
+
+    def compute_features_compacity(self, selection, distance, nb_features):
+        """
+        For a selection of instances, compute features compacity metrics used in method `compacity_plot`.
+
+        The method returns :
+        * the minimum number of features needed for a given approximation level
+        * conversely, the approximation reached with a given number of features
+
+        Parameters
+        ----------
+        selection: list
+            Indices of rows to be displayed on the stability plot
+        distance : float
+            How close we want to be from model with all features
+        nb_features : int
+            Number of features used
+        """
+        if (self._case == "classification") and (len(self._classes) > 2):
+            raise AssertionError("Multi-class classification is not supported")
+
+        features_needed = get_min_nb_features(selection, self.contributions, self._case, distance)
+        distance_reached = get_distance(selection, self.contributions, self._case, nb_features)
+        # We clip large approximations to 100%
+        distance_reached = np.clip(distance_reached, 0, 1)
+
+        self.features_compacity = {"features_needed": features_needed, "distance_reached": distance_reached}
+
     def init_app(self, settings: dict = None):
         """
         Simple init of SmartApp in case of host smartapp by another way
@@ -1239,23 +1328,29 @@ class SmartExplainer:
             raise AssertionError("Explainer object was not compiled. Please compile the explainer "
                                  "object using .compile(...) method before generating the report.")
 
-        execute_report(
-            working_dir=working_dir,
-            explainer=self,
-            project_info_file=project_info_file,
-            x_train=x_train,
-            y_train=y_train,
-            y_test=y_test,
-            config=dict(
-                title_story=title_story,
-                title_description=title_description,
-                metrics=metrics
-            ),
-            notebook_path=notebook_path,
-            kernel_name=kernel_name
-        )
-        export_and_save_report(working_dir=working_dir, output_file=output_file)
+        try:
+            execute_report(
+                working_dir=working_dir,
+                explainer=self,
+                project_info_file=project_info_file,
+                x_train=x_train,
+                y_train=y_train,
+                y_test=y_test,
+                config=dict(
+                    title_story=title_story,
+                    title_description=title_description,
+                    metrics=metrics
+                ),
+                notebook_path=notebook_path,
+                kernel_name=kernel_name
+            )
+            export_and_save_report(working_dir=working_dir, output_file=output_file)
 
-        if rm_working_dir:
-            shutil.rmtree(working_dir)
+            if rm_working_dir:
+                shutil.rmtree(working_dir)
+        
+        except Exception as e:
+            if rm_working_dir:
+                shutil.rmtree(working_dir)
+            raise e
 
